# -*- coding: utf-8 -*-
import tornado.ioloop
import tornado.web
import tornado.gen
import tornado.concurrent
import json
import datetime
import time
import os
import sys
import traceback
import pymongo
from pymongo import MongoClient


class Logger(object):
    def __init__(self):
        self.terminal = sys.stdout
        self.log = open("./log/app.log", "a")

    def write(self, message):
        self.terminal.write(message)
        self.log.write(message)

    def flush(self):
        self.terminal.flush()
        self.log.flush()

sys.stdout = Logger()

class HealthHandler(tornado.web.RequestHandler):
    def set_default_headers(self):
        self.set_header("Content-Type", 'text/plain')

    @tornado.gen.coroutine
    def get(self):
        self.write('Service is running!')
        self.finish()

class  InsertDataHandler(tornado.web.RequestHandler):
    def initialize(self,db):
         self.db = db

    def set_default_headers(self):
        self.set_header("Content-Type", 'application/json')

    @tornado.gen.coroutine
    def post(self,*args):
        start_time = time.time()
        print('Insertdata: new request /restfuldatalogger/insertdata {}'
              .format(str(datetime.datetime.now())))

        data_dict = self.process_request()
        data_dict['TimestampInsertion'] = datetime.datetime.now()

        dbcl = self.db[data_dict['id']]
        x = dbcl.insert_one(data_dict)

        print('Insertdata: data inserted--- {} ---'.format(x.inserted_id))
        print('Insertdata: execution time--- {} seconds ---'
              .format((time.time() - start_time)))

        self.write('{"Result" : "Success"}')
        self.finish()

    def process_request(self):
        data_dict = {'id' : '', 'value1' : '', 'value2' : '', 'TimestampAcquisition' : ''}

        try:
            body = self.request.body.decode()
            if (body != ''):
                content = json.loads(body)
                data_dict['id'] = str(content['id'])
                data_dict['value1'] = int(content['value1'])
                data_dict['value2'] = int(content['value2'])
                data_dict['TimestampAcquisition'] =  \
                datetime.datetime.strptime(content['TimestampAcquisition'], '%Y-%m-%d %H:%M:%S')
            else:
                print('Insertdata: empty body')
                raise tornado.web.HTTPError(400)
        except Exception as e:
            print('Insertdata: error in parsing body')
            print(e)
            print(traceback.print_tb(e.__traceback__))
            raise tornado.web.HTTPError(400)

        return data_dict

class  ReadDataHandler(tornado.web.RequestHandler):
    def initialize(self, db):
        self.db = db

    def set_default_headers(self):
        self.set_header("Content-Type", 'application/json')

    @tornado.gen.coroutine
    def get(self,*args):
        start_time = time.time()
        print('ReadData: new request /restfuldatalogger/readdata {}'
              .format(str(datetime.datetime.now())))
        print('ReadData: reading data for id {}'.format(args[0]))

        dbcl = self.db[args[0]]
        cursor = dbcl.find({}, sort=[('_id', pymongo.DESCENDING)]).limit(10)
        arrayvalue1=[]
        arrayvalue2=[]
        for document in cursor:
            arrayvalue1.append(document["value1"])
            arrayvalue2.append(document["value2"])

        data={"id" : args[0], "value1" : arrayvalue1, "value2" : arrayvalue2}
        response = json.dumps(data)
        print('ReadData: execution time--- {} seconds ---'
              .format((time.time() - start_time)))

        self.write(response)
        self.finish()

def make_app(db):

    app = tornado.web.Application([
        (r"/health", HealthHandler),
        (r"/restfuldatalogger/insertdata", InsertDataHandler, {'db': db}),
        (r"/restfuldatalogger/readdata/([0-9]+)", ReadDataHandler, {'db': db})
    ])

    return app

if __name__ == "__main__":

    print('Start application {}'.format(datetime.datetime.now()))

    TORNADO_PORT = int(os.environ.get("APP_PORT", 8890))
    MONGODB_HOSTNAME = os.environ.get("MONGODB_HOSTNAME", 'localhost')
    MONGODB_PORT = int(os.environ.get("MONGODB_PORT", 27017))
    MONGODB_DATABASE=os.environ.get("MONGODB_DATABASE", 'appdb')
    MONGODB_USERNAME=os.environ.get("MONGODB_USERNAME", '')
    MONGODB_PASSWORD=os.environ.get("MONGODB_PASSWORD", '')

    try:
        dbclient = MongoClient(host=MONGODB_HOSTNAME,
                               port=MONGODB_PORT,
                               username=MONGODB_USERNAME,
                               password=MONGODB_PASSWORD,
                               authSource=MONGODB_DATABASE,
                               serverSelectionTimeoutMS=1)
        dbclient.server_info()
        db = dbclient[MONGODB_DATABASE]

        print('Application running on port {}'.format(TORNADO_PORT))
        app = make_app(db)
        app.listen(TORNADO_PORT)
        tornado.ioloop.IOLoop.current().start()

    except Exception as e:
        print('Application error: {}' .format(e))
